@@ -80,9 +80,8 @@ class GeneratorTestCase(base.BaseTestCase):
                                   default='a',
                                   choices=(None, '', 'a', 'b', 'c'),
                                   help='a string with choices'),
-        'deprecated_opt': cfg.StrOpt('bar',
-                                     deprecated_name='foobar',
-                                     help='deprecated'),
+        'deprecated_opt_without_deprecated_group': cfg.StrOpt(
+            'bar', deprecated_name='foobar', help='deprecated'),
         'deprecated_for_removal_opt': cfg.StrOpt(
             'bar', deprecated_for_removal=True, help='deprecated for removal'),
         'deprecated_reason_opt': cfg.BoolOpt(
@@ -91,11 +90,10 @@ class GeneratorTestCase(base.BaseTestCase):
             deprecated_for_removal=True,
             deprecated_reason='This was supposed to work but it really, '
                               'really did not. Always buy house insurance.',
-            help='Turn off stove'),
-        'deprecated_group': cfg.StrOpt('bar',
-                                       deprecated_group='group1',
-                                       deprecated_name='foobar',
-                                       help='deprecated'),
+            help='DEPRECATED: Turn off stove'),
+        'deprecated_opt_with_deprecated_group': cfg.StrOpt(
+            'bar', deprecated_name='foobar', deprecated_group='group1',
+            help='deprecated'),
         # Unknown Opt default must be a string
         'unknown_type': cfg.Opt('unknown_opt',
                                 default='123',
@@ -398,8 +396,10 @@ class GeneratorTestCase(base.BaseTestCase):
 # Allowed values: <None>, '', a, b, c
 #choices_opt = a
 ''')),
-        ('deprecated',
-         dict(opts=[('test', [(groups['foo'], [opts['deprecated_opt']])])],
+        ('deprecated opt without deprecated group',
+         dict(opts=[('test',
+                     [(groups['foo'],
+                       [opts['deprecated_opt_without_deprecated_group']])])],
               expected='''[DEFAULT]
 
 
@@ -411,7 +411,7 @@ class GeneratorTestCase(base.BaseTestCase):
 #
 
 # deprecated (string value)
-# Deprecated group/name - [DEFAULT]/foobar
+# Deprecated group/name - [foo]/foobar
 #bar = <None>
 ''')),
         ('deprecated_for_removal',
@@ -427,7 +427,7 @@ class GeneratorTestCase(base.BaseTestCase):
 # From test
 #
 
-# deprecated for removal (string value)
+# DEPRECATED: deprecated for removal (string value)
 # This option is deprecated for removal.
 # Its value may be silently ignored in the future.
 #bar = <None>
@@ -445,15 +445,17 @@ class GeneratorTestCase(base.BaseTestCase):
 # From test
 #
 
-# Turn off stove (boolean value)
+# DEPRECATED: Turn off stove (boolean value)
 # This option is deprecated for removal.
 # Its value may be silently ignored in the future.
 # Reason: This was supposed to work but it really, really did not.
 # Always buy house insurance.
 #turn_off_stove = false
 ''')),
-        ('deprecated_group',
-         dict(opts=[('test', [(groups['foo'], [opts['deprecated_group']])])],
+        ('deprecated_opt_with_deprecated_group',
+         dict(opts=[('test',
+                     [(groups['foo'],
+                       [opts['deprecated_opt_with_deprecated_group']])])],
               expected='''[DEFAULT]
 
 
@@ -710,6 +712,78 @@ class GeneratorTestCase(base.BaseTestCase):
 
 # string with bad default (string value)
 #string_type_with_bad_default = 4096
+''')),
+         ('str_opt_str_group',
+         dict(opts=[('test', [('foo',
+                               [opts['str_opt']]),
+                              (groups['foo'],
+                               [opts['int_opt']])]),
+                    ('foo', [('foo',
+                               [opts['bool_opt']])])],
+              expected='''[DEFAULT]
+
+
+[foo]
+# foo help
+
+#
+# From foo
+#
+
+# a boolean (boolean value)
+#bool_opt = false
+
+#
+# From test
+#
+
+# a string (string value)
+#str_opt = foo bar
+
+#
+# From test
+#
+
+# an integer (integer value)
+# Minimum value: 1
+# Maximum value: 20
+#int_opt = 10
+''')),
+         ('opt_str_opt_group',
+         dict(opts=[('test', [(groups['foo'],
+                               [opts['int_opt']]),
+                              ('foo',
+                               [opts['str_opt']])]),
+                    ('foo', [(groups['foo'],
+                              [opts['bool_opt']])])],
+              expected='''[DEFAULT]
+
+
+[foo]
+# foo help
+
+#
+# From foo
+#
+
+# a boolean (boolean value)
+#bool_opt = false
+
+#
+# From test
+#
+
+# an integer (integer value)
+# Minimum value: 1
+# Maximum value: 20
+#int_opt = 10
+
+#
+# From test
+#
+
+# a string (string value)
+#str_opt = foo bar
 ''')),
     ]
 
@@ -782,8 +856,9 @@ class GeneratorTestCase(base.BaseTestCase):
         if self.stdout:
             self.assertEqual(self.expected, stdout.getvalue())
         else:
-            content = open(output_file).read()
-            self.assertEqual(self.expected, content)
+            with open(output_file, 'r') as f:
+                actual = f.read()
+            self.assertEqual(self.expected, actual)
 
         log_warning = getattr(self, 'log_warning', None)
         if log_warning is not None:
@@ -869,19 +944,10 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
             cfg.BoolOpt('bool_opt', help='a boolean'),
             cfg.IntOpt('int_opt', help='an integer')]
 
-    def test_get_group_name(self):
-        name = "group1"
-        item = [name]
-        self.assertEqual(name, generator._get_group_name(item))
-
-    def test_get_group_name_as_optgroup(self):
-        name = "group2"
-        item = [cfg.OptGroup(name)]
-        self.assertEqual(name, generator._get_group_name(item))
-
     def test_get_groups_empty_ns(self):
         groups = generator._get_groups([])
-        self.assertEqual({'DEFAULT': []}, groups)
+        self.assertEqual({'DEFAULT': {'object': None, 'namespaces': []}},
+                         groups)
 
     def test_get_groups_single_ns(self):
         config = [("namespace1", [
@@ -907,14 +973,14 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
         groups = generator._get_groups(config)
 
         fd, tmp_file = tempfile.mkstemp()
-        f = open(tmp_file, 'w+')
-        formatter = generator._OptFormatter(output_file=f)
+        with open(tmp_file, 'w+') as f:
+            formatter = generator._OptFormatter(output_file=f)
+            generator._output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'))
         expected = '''[DEFAULT]
 '''
-        generator._output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'))
-        f.close()
-        content = open(tmp_file).read()
-        self.assertEqual(expected, content)
+        with open(tmp_file, 'r') as f:
+            actual = f.read()
+        self.assertEqual(expected, actual)
 
     def test_output_opts_group(self):
 
@@ -922,6 +988,29 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
                    ("alpha", [self.opts[0]])])]
         groups = generator._get_groups(config)
 
+        fd, tmp_file = tempfile.mkstemp()
+        with open(tmp_file, 'w+') as f:
+            formatter = generator._OptFormatter(output_file=f)
+            generator._output_opts(formatter, 'alpha', groups.pop('alpha'))
+        expected = '''[alpha]
+
+#
+# From namespace1
+#
+
+# foo option (string value)
+#foo = fred
+'''
+        with open(tmp_file, 'r') as f:
+            actual = f.read()
+        self.assertEqual(expected, actual)
+
+    def _test_output_default_list_opt_with_string_value(self, default):
+        opt = cfg.ListOpt('list_opt', help='a list', default=default)
+        config = [("namespace1", [
+                   ("alpha", [opt])])]
+        groups = generator._get_groups(config)
+
         fd, tmp_file = tempfile.mkstemp()
         f = open(tmp_file, 'w+')
         formatter = generator._OptFormatter(output_file=f)
@@ -931,14 +1020,45 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
 # From namespace1
 #
 
-# foo option (string value)
-#foo = fred
-'''
+# a list (list value)
+#list_opt = %(default)s
+''' % {'default': default}
         generator._output_opts(formatter, 'alpha', groups.pop('alpha'))
         f.close()
         content = open(tmp_file).read()
         self.assertEqual(expected, content)
 
+    def test_output_default_list_opt_with_string_value_multiple_entries(self):
+        self._test_output_default_list_opt_with_string_value('foo,bar')
+
+    def test_output_default_list_opt_with_string_value_single_entry(self):
+        self._test_output_default_list_opt_with_string_value('foo')
+
+
+class GeneratorMutableOptionTestCase(base.BaseTestCase):
+
+    def test_include_message(self):
+        out = moves.StringIO()
+        opt = cfg.StrOpt('foo', help='foo option', mutable=True)
+        gen = generator._OptFormatter(output_file=out)
+        gen.format(opt, 'group1')
+        result = out.getvalue()
+        self.assertIn(
+            'This option can be changed without restarting.',
+            result,
+        )
+
+    def test_do_not_include_message(self):
+        out = moves.StringIO()
+        opt = cfg.StrOpt('foo', help='foo option', mutable=False)
+        gen = generator._OptFormatter(output_file=out)
+        gen.format(opt, 'group1')
+        result = out.getvalue()
+        self.assertNotIn(
+            'This option can be changed without restarting.',
+            result,
+        )
+
 
 class GeneratorRaiseErrorTestCase(base.BaseTestCase):
 
